@@ -95,6 +95,7 @@ class Workspace(Registrable):
         return out
 
     @property
+    @abstractmethod
     def url(self) -> str:
         """
         Get a URL for the workspace that can be used to instantiate the same workspace
@@ -157,7 +158,10 @@ class Workspace(Registrable):
         """
         Returns a :class:`~tango.step_info.StepInfo` for a given step.
 
-        :raises KeyError: If the step has not been registered as part of a run yet.
+        :raises KeyError: If the corresponding step info cannot be found or created.
+            This should never happen if you pass a :class:`~tango.step.Step` object to this method
+            since a :class:`~tango.step_info.StepInfo` can always be created from a
+            :class:`~tango.step.Step`.
         """
         raise NotImplementedError()
 
@@ -167,6 +171,8 @@ class Workspace(Registrable):
         The :class:`.Step` class calls this when a step is about to start running.
 
         :param step: The step that is about to start.
+
+        :raises StepStateError: If the step is in an unexpected state (e.g. RUNNING).
         """
         raise NotImplementedError()
 
@@ -177,6 +183,8 @@ class Workspace(Registrable):
 
         :param step: The step that finished.
 
+        :raises StepStateError: If the step is in an unexpected state (e.g. RUNNING).
+
         This method is given the result of the step's :meth:`.Step.run` method. It is expected to return that
         result. This gives it the opportunity to make changes to the result if necessary. For example, if the
         :meth:`.Step.run` method returns an iterator, that iterator would be consumed when it's written to the
@@ -191,6 +199,8 @@ class Workspace(Registrable):
 
         :param step: The step that failed.
         :param e: The exception thrown by the step's :meth:`.Step.run` method.
+
+        :raises StepStateError: If the step is in an unexpected state (e.g. RUNNING).
         """
         raise NotImplementedError()
 
